% primo test di applicazione della cross-validation (CV), degli indici
% Final Prediction Error (FPE), di Akaike (AIC) e Minimum Description
% Length (MDL) per determinare classe e complessit? di un modello.
%
% per semplicita' e per ottenere risultati velocemente, vengono testati solo
% pochi modelli della classe ARX, in particolare modelli ARX(n,n) n=1, 2,
% 3, 4...10
% Come risultato vengono generati grafici dell'andamento dei 4 indici (CV,
% FPE, AIK, MDL).
%
% Si noti come nel caso considerato tutti gli indici indichino il modello
% ARX(2,2) come migliore candidato.
%
% Testato in Matlab R2012a
%
% corso "Sistemi Dinamici" 2019/2020
% G. Fenu
%



clear 
close all

%------- Creazione dei dati ------
ndati=20000;
y=zeros(ndati,1);
y(1)=randn;y(2)=randn;
err=1*randn(size(y));
u=2*randn(size(y));
t=(0:.05:.05*ndati-.05)';

% ARX(2,2) 
for i=3:length(y)
    y(i)=1.2*y(i-1)-0.32*y(i-2)+u(i-1)+0.5*u(i-2)+err(i);
end

figure;
plot(y)
hold on
plot(u,'r')
legend('uscita','ingresso manipolabile');
set(gca, 'FontSize', 14, 'FontWeight', 'bold');


%------ Ripartizione dati per crossvalidazione --------

traindata=round(60*ndati/100);
ytrain=y(1:traindata);
utrain=u(1:traindata);
ytest=y(traindata+1:end);
utest=u(traindata+1:end);
datitest=iddata(ytest,utest);
datitrain=iddata(ytrain,utrain);
N=length(ytest);
NT=length(ytrain);

JCV=zeros(10,1);
JTR=zeros(10,1);

for n=1:10
    
    model=arx([ytrain utrain],[n n 1]);
    
    % uso i dati di addestramento
        prederr=pe(model,datitrain);
        
        err=prederr.OutputData;
    %--- Errore quadratico medio commesso ---
        JTR(n)=1/NT*err'*err;% SUI DATI DI TRAIN
    
    clear prederr err
    %----- Validazione ------
        prederr=pe(model,datitest);
        
        err=prederr.OutputData;
    %--- Errore quadratico medio commesso ---
        JCV(n)=1/N*(err'*err); % sui dati di validazione
      
end

Fpe=(N+(2:2:20)')./(N-(2:2:20)').*JTR;
Akaike=2*(2:2:20)'/N+log(JTR); %  J calcolato sul train 
Mdl=log(N)*(2:2:20)'/N+log(JTR);

figure
plot(2:2:20,Fpe, 'LineWidth', 2)
hold on
plot(2:2:20,Akaike,'r', 'LineWidth', 2)
plot(2:2:20,Mdl,'k', 'LineWidth', 2)
plot(2:2:20,JCV,'m--', 'LineWidth', 2);
legend('Final prediction error','Akaike','Minimum description length','CROSS-VALIDATION')
xlabel('Ordine complessivo (n_a+n_b)')
set(gca, 'FontSize', 14, 'FontWeight', 'bold');
grid on